/**
 * Module responsible for preformance improvements when browsing / scrolling data.
 *
 * The main task of this module is to generate an optimization structure of Map<number, number> where:
 * a) key - unix timestamp,
 * b) value - index indicating a point in array of allPoints: IUnixTimePoint[]
 *
 * If:
 * a) quite large data series are loaded, there are no additional,
 * "ahead of time data" structures created - only ad-hoc buckets;
 * note there as many buckets as there are on screen pixel columns on SVG time series chart.
 *
 * b) the view is narrowed to see only a fraction of the series,
 * ad-hoc buckets must be created; it is good to know from which index to
 * start from when building buckets for a given unix timestap.
 */
import * as _ from "lodash";
export var MAX_UNIX_INDEX_MAP_BUCKET_SIZE = 10000;
var calculateMapParameters = function (allPoints) {
    var length = MAX_UNIX_INDEX_MAP_BUCKET_SIZE;
    return {
        unixFrom: _.isObject(_.first(allPoints)) ? _.first(allPoints).unix : 0,
        length: length,
        bucketLengthUnix: (_.last(allPoints).unix - _.first(allPoints).unix) / length
    };
};
/**
 * Return an index in allPoints array, indicating an existing point p: IUnixTimePoint, meeting two conditions:
 * a) it is available as a value in pre-created unixToIndexMap (key, value),
 * b) the point found in allPoints array is the first one having
 *    p.unix >= browseFromUnix
 * @param allPoints all data available
 * @param unixToIndexMap pre-calculated map
 * @param browseFromUnix
 */
var findIndexToBrowsePointsFrom = function (allPoints, unixToIndexMap, browseFromUnix) {
    var mapParameters = calculateMapParameters(allPoints);
    var unix = mapParameters.unixFrom + _.floor((browseFromUnix - mapParameters.unixFrom) /
        mapParameters.bucketLengthUnix) * mapParameters.bucketLengthUnix;
    return unixToIndexMap.get(unix) || 0;
};
var findFirstIndexMeetingUnixFrom = function (allPoints, unixFrom, lastIndex) {
    for (var i = lastIndex; i < allPoints.length - 1; i++) {
        if (allPoints[i].unix >= unixFrom)
            return i;
    }
    return 0;
};
/**
 * Creates a Map used for browse/scroll speed optimizations
 */
var createUnixToIndexMap = function (allPoints) {
    var result = new Map();
    if (allPoints.length == 0)
        return result;
    var parameters = calculateMapParameters(allPoints);
    var firstUnixFrom = _.first(allPoints).unix;
    var indexOfElement = 0;
    for (var i = 0; i < parameters.length; i++) {
        var unixFrom = firstUnixFrom + i * parameters.bucketLengthUnix;
        var unixTo = firstUnixFrom + (i + 1) * parameters.bucketLengthUnix;
        indexOfElement = findFirstIndexMeetingUnixFrom(allPoints, unixFrom, indexOfElement);
        result.set(unixFrom, indexOfElement);
    }
    return result;
};
export var unixIndexMapCalculations = {
    createUnixToIndexMap: createUnixToIndexMap,
    findIndexToBrowsePointsFrom: findIndexToBrowsePointsFrom
};
//# sourceMappingURL=unix-index-map.js.map